@@ -1,11 +1,3 @@
-from __future__ import absolute_import
-
-try:
-    import itertools.izip as zip
-except ImportError:
-    pass
-
-import sys
 import types
 import copy
 
@@ -16,7 +8,7 @@ import numpy as np
 import param
 import pandas as pd # noqa
 
-from param.parameterized import add_metaclass, ParameterizedMetaclass
+from param.parameterized import ParameterizedMetaclass
 
 from ..accessors import Redim
 from ..dimension import (
@@ -73,7 +65,7 @@ def concat(datasets, datatype=None):
     return Interface.concatenate(datasets, datatype)
 
 
-class DataConversion(object):
+class DataConversion:
     """
     DataConversion is a very simple container object which can be
     given an existing Dataset Element and provides methods to convert
@@ -132,7 +124,7 @@ class DataConversion(object):
             else:
                 selected = self._element
         else:
-            if pd and issubclass(self._element.interface, PandasInterface):
+            if issubclass(self._element.interface, PandasInterface):
                 ds_dims = self._element.dimensions()
                 ds_kdims = [self._element.get_dimension(d) if d in ds_dims else d
                             for d in groupby+kdims]
@@ -244,8 +236,7 @@ class PipelineMeta(ParameterizedMetaclass):
         return pipelined_fn
 
 
-@add_metaclass(PipelineMeta)
-class Dataset(Element):
+class Dataset(Element, metaclass=PipelineMeta):
     """
     Dataset provides a general baseclass for Element types that
     contain structured data and supports a range of data formats.
@@ -286,20 +277,19 @@ class Dataset(Element):
         """
         if isinstance(data, DynamicMap):
             class_name = cls.__name__
-            repr_kdims = 'kdims=%r' % kdims if kdims else None
-            repr_vdims = 'vdims=%r' % vdims if vdims else None
-            repr_kwargs = (', '.join('%s=%r' % (k,v) for k,v in kwargs.items())
+            repr_kdims = f'kdims={kdims!r}' if kdims else None
+            repr_vdims = f'vdims={vdims!r}' if vdims else None
+            repr_kwargs = (', '.join(f'{k}={v!r}' for k,v in kwargs.items())
                            if kwargs else None)
             extras = ', '.join([el for el in [repr_kdims, repr_vdims, repr_kwargs]
                                if el is not None])
             extras = ', ' + extras if extras else ''
-            apply_args= 'hv.{class_name}{extras}'.format(class_name=class_name,
-                                                         extras=extras)
+            apply_args= f'hv.{class_name}{extras}'
             msg = "Cannot construct a {class_name} from the supplied object of type DynamicMap. Implicitly creating a DynamicMap of {class_name} objects, but instead please explicitly call .apply({apply_args}) on the supplied DynamicMap."
             cls.param.warning(cls, msg.format(class_name=class_name, apply_args=apply_args))
             return data.apply(cls, per_element=True, kdims=kdims, vdims=vdims, **kwargs)
         else:
-            return super(Dataset, cls).__new__(cls)
+            return super().__new__(cls)
 
     def __init__(self, data, kdims=None, vdims=None, **kwargs):
         from ...operation.element import (
@@ -315,12 +305,12 @@ class Dataset(Element):
         if isinstance(data, Element):
             if 'kdims' in kwargs:
                 kwargs['kdims'] = [
-                    data.get_dimension(kd) if isinstance(kd, util.basestring) else kd
+                    data.get_dimension(kd) if isinstance(kd, str) else kd
                     for kd in kwargs['kdims']
                 ]
             if 'kdims' in kwargs:
                 kwargs['vdims'] = [
-                    data.get_dimension(vd) if isinstance(vd, util.basestring) else vd
+                    data.get_dimension(vd) if isinstance(vd, str) else vd
                     for vd in kwargs['vdims']
                 ]
             pvals = util.get_param_values(data)
@@ -341,7 +331,7 @@ class Dataset(Element):
         initialized = Interface.initialize(type(self), data, kdims, vdims,
                                            datatype=kwargs.get('datatype'))
         (data, self.interface, dims, extra_kws) = initialized
-        super(Dataset, self).__init__(data, **dict(kwargs, **dict(dims, **extra_kws)))
+        super().__init__(data, **dict(kwargs, **dict(dims, **extra_kws)))
         self.interface.validate(self, validate_vdims)
 
         # Handle _pipeline property
@@ -378,7 +368,7 @@ class Dataset(Element):
 
     def __getstate__(self):
         "Ensures pipelines are dropped"
-        obj_dict = super(Dataset, self).__getstate__()
+        obj_dict = super().__getstate__()
         if '_pipeline' in obj_dict:
             pipeline = obj_dict['_pipeline']
             obj_dict['_pipeline'] = pipeline.instance(operations=pipeline.operations[:1])
@@ -539,11 +529,11 @@ class Dataset(Element):
         Returns:
             Cloned object containing the new dimension
         """
-        if isinstance(dimension, (util.basestring, tuple)):
+        if isinstance(dimension, (str, tuple)):
             dimension = Dimension(dimension)
 
         if dimension.name in self.kdims:
-            raise Exception('{dim} dimension already defined'.format(dim=dimension.name))
+            raise Exception(f'{dimension.name} dimension already defined')
 
         if vdim:
             dims = self.vdims[:]
@@ -719,7 +709,7 @@ argument to specify a selection specification""")
             value_select = slices[self.ndims]
         elif len(slices) == self.ndims+1 and isinstance(slices[self.ndims],
                                                         (Dimension,str)):
-            raise IndexError("%r is not an available value dimension" % slices[self.ndims])
+            raise IndexError(f"{slices[self.ndims]!r} is not an available value dimension")
         else:
             selection = dict(zip(self.dimensions(label=True), slices))
         data = self.select(**selection)
@@ -798,9 +788,9 @@ argument to specify a selection specification""")
 
         # Note: Special handling sampling of gridded 2D data as Curve
         # may be replaced with more general handling
-        # see https://github.com/ioam/holoviews/issues/1173
+        # see https://github.com/holoviz/holoviews/issues/1173
         from ...element import Table, Curve
-        datatype = ['dataframe', 'dictionary', 'dask', 'ibis']
+        datatype = ['dataframe', 'dictionary', 'dask', 'ibis', 'cuDF']
         if len(samples) == 1:
             sel = {kd.name: s for kd, s in zip(self.kdims, samples[0])}
             dims = [kd for kd, v in sel.items() if not np.isscalar(v)]
@@ -824,7 +814,7 @@ argument to specify a selection specification""")
             return self.clone(selection, kdims=kdims, new_type=new_type,
                               datatype=datatype)
 
-        lens = set(len(util.wrap_tuple(s)) for s in samples)
+        lens = {len(util.wrap_tuple(s)) for s in samples}
         if len(lens) > 1:
             raise IndexError('Sample coordinates must all be of the same length.')
 
@@ -949,7 +939,7 @@ argument to specify a selection specification""")
             try:
                 # Should be checking the dimensions declared on the element are compatible
                 return self.clone(aggregated, kdims=kdims, vdims=vdims)
-            except:
+            except Exception:
                 datatype = self.param.objects('existing')['datatype'].default
                 return self.clone(aggregated, kdims=kdims, vdims=vdims,
                                   new_type=new_type, datatype=datatype)
@@ -1073,12 +1063,10 @@ argument to specify a selection specification""")
         "Number of values in the Dataset."
         return self.interface.length(self)
 
-    def __nonzero__(self):
+    def __bool__(self):
         "Whether the Dataset contains any values"
         return self.interface.nonzero(self)
 
-    __bool__ = __nonzero__
-
     @property
     def shape(self):
         "Returns the shape of the data."
@@ -1105,8 +1093,6 @@ argument to specify a selection specification""")
         values = self.interface.values(self, dim, expanded, flat)
         if dim.nodata is not None:
             # Ensure nodata applies to boolean data in py2
-            if sys.version_info.major == 2 and values.dtype.kind == 'b':
-                values = values.astype('int')
             values = np.where(values==dim.nodata, np.NaN, values)
         return values
 
@@ -1208,23 +1194,21 @@ argument to specify a selection specification""")
         elif self._in_method and 'dataset' not in overrides:
             overrides['dataset'] = self.dataset
 
-        return super(Dataset, self).clone(
-            data, shared_data, new_type, *args, **overrides
-        )
+        return super().clone(data, shared_data, new_type, *args, **overrides)
 
     # Overrides of superclass methods that are needed so that PipelineMeta
     # will find them to wrap with pipeline support
     @wraps(Dimensioned.options)
     def options(self, *args, **kwargs):
-        return super(Dataset, self).options(*args, **kwargs)
+        return super().options(*args, **kwargs)
 
     @wraps(LabelledData.map)
     def map(self, *args, **kwargs):
-        return super(Dataset, self).map(*args, **kwargs)
+        return super().map(*args, **kwargs)
 
     @wraps(LabelledData.relabel)
     def relabel(self, *args, **kwargs):
-        return super(Dataset, self).relabel(*args, **kwargs)
+        return super().relabel(*args, **kwargs)
 
     @property
     def iloc(self):
