@@ -2,11 +2,10 @@ import numpy as np
 import param
 
 from ..core import util
-from ..core import Dimension, Dataset, Element2D
-from ..core.data import GridInterface
-from ..streams import SelectionXY
+from ..core import Dimension, Dataset, Element2D, NdOverlay, Overlay
+from ..core.dimension import process_dimensions
 from .geom import Rectangles, Points, VectorField # noqa: backward compatible import
-from .selection import Selection1DExpr, Selection2DExpr
+from .selection import Selection1DExpr
 
 
 class Chart(Dataset, Element2D):
@@ -42,13 +41,20 @@ class Chart(Dataset, Element2D):
     # Enables adding index if 1D array like data is supplied
     _auto_indexable_1d = True
 
+    _max_kdim_count = 1 # Remove once kdims has bounds=(1,1) instead of warning
     __abstract = True
 
+    def __init__(self, data, kdims=None, vdims=None, **params):
+        params.update(process_dimensions(kdims, vdims))
+        if len(params.get('kdims', [])) == self._max_kdim_count + 1:
+            self.param.warning('Chart elements should only be supplied a single kdim')
+        super().__init__(data, **params)
+
     def __getitem__(self, index):
-        return super(Chart, self).__getitem__(index)
+        return super().__getitem__(index)
 
 
-class Scatter(Selection2DExpr, Chart):
+class Scatter(Selection1DExpr, Chart):
     """
     Scatter is a Chart element representing a set of points in a 1D
     coordinate system where the key dimension maps to the points
@@ -127,8 +133,7 @@ class ErrorBars(Selection1DExpr, Chart):
             if not dimension_range:
                 return (lower, upper)
             return util.dimension_range(lower, upper, dim.range, dim.soft_range)
-        return super(ErrorBars, self).range(dim, data_range)
-
+        return super().range(dim, data_range)
 
 
 class Spread(ErrorBars):
@@ -143,8 +148,7 @@ class Spread(ErrorBars):
     group = param.String(default='Spread', constant=True)
 
 
-
-class Bars(Chart):
+class Bars(Selection1DExpr, Chart):
     """
     Bars is a Chart element representing categorical observations
     using the height of rectangular bars. The key dimensions represent
@@ -157,9 +161,10 @@ class Bars(Chart):
 
     kdims = param.List(default=[Dimension('x')], bounds=(1,3))
 
+    _max_kdim_count = 3
 
 
-class Histogram(Chart):
+class Histogram(Selection1DExpr, Chart):
     """
     Histogram is a Chart element representing a number of bins in a 1D
     coordinate system. The key dimension represents the binned values,
@@ -180,138 +185,14 @@ class Histogram(Chart):
 
     _binned = True
 
-    _selection_streams = (SelectionXY,)
-
-    def __init__(self, data, edges=None, **params):
+    def __init__(self, data, **params):
         if data is None:
             data = []
-        if edges is not None:
-            self.param.warning(
-                "Histogram edges should be supplied as a tuple "
-                "along with the values, passing the edges will "
-                "be deprecated in holoviews 2.0.")
-            data = (edges, data)
-        elif isinstance(data, tuple) and len(data) == 2 and len(data[0])+1 == len(data[1]):
+        if (isinstance(data, tuple) and len(data) == 2 and
+            len(data[0])+1 == len(data[1])):
             data = data[::-1]
 
-        super(Histogram, self).__init__(data, **params)
-
-    def _get_selection_expr_for_stream_value(self, **kwargs):
-        from ..util.transform import dim
-
-        invert_axes = self.opts.get('plot').kwargs.get('invert_axes', False)
-
-        ds = self.dataset
-        if kwargs.get('bounds', None) is None:
-            el = ds.clone([]) if ds.interface.gridded else ds.iloc[:0]
-            return None, None, self.pipeline(el)
-
-        if invert_axes:
-            y0, x0, y1, x1 = kwargs['bounds']
-        else:
-            x0, y0, x1, y1 = kwargs['bounds']
-
-        # Handle invert_xaxis/invert_yaxis
-        if y0 > y1:
-            y0, y1 = y1, y0
-        if x0 > x1:
-            x0, x1 = x1, x0
-
-        xdim = self.kdims[0]
-        ydim = self.vdims[0]
-
-        edges = self.edges
-        centers = self.dimension_values(xdim)
-        heights = self.dimension_values(ydim)
-
-        selected_mask = (
-            (centers >= x0) & (centers <= x1) &
-            (heights >= y0) & (heights <= y1)
-        )
-
-        selected_bins = (np.arange(len(centers))[selected_mask] + 1).tolist()
-        if not selected_bins:
-            el = ds.clone([]) if ds.interface.gridded else ds.iloc[:0]
-            return None, None, self.pipeline(el)
-
-        bbox = {
-            xdim.name: (
-                edges[max(0, min(selected_bins) - 1)],
-                edges[min(len(edges - 1), max(selected_bins))],
-            ),
-        }
-        index_cols = kwargs.get('index_cols')
-        if index_cols:
-            shape = dim(self.dataset.get_dimension(index_cols[0]), np.shape)
-            index_cols = [dim(self.dataset.get_dimension(c), np.ravel) for c in index_cols]
-            sel = self.dataset.clone(datatype=['dataframe', 'dictionary']).select(**bbox)
-            vals = dim(index_cols[0], util.unique_zip, *index_cols[1:]).apply(
-                sel, expanded=True, flat=True
-            )
-            contains = dim(index_cols[0], util.lzip, *index_cols[1:]).isin(vals, object=True)
-            selection_expr = dim(contains, np.reshape, shape)
-            region = None
-        else:
-            selection_expr = dim(xdim).digitize(edges).isin(selected_bins)
-            if selected_bins[-1] == len(centers):
-                # Handle values exactly on the upper boundary
-                selection_expr = selection_expr | (dim(xdim) == edges[-1])
-            if ds.interface.gridded:
-                mask = selection_expr.apply(ds, expanded=True, flat=False)
-                ds = ds.clone(ds.interface.mask(ds, mask))
-            else:
-                ds = ds.select(selection_expr)
-            region = self.pipeline(ds)
-        return selection_expr, bbox, region
-
-    @staticmethod
-    def _merge_regions(region1, region2, operation):
-        if region1 is None:
-            if operation == 'inverse':
-                return region2.clone(data=(
-                    (region2.edges,
-                     np.zeros_like(region2.dimension_values(1)))
-                ))
-            else:
-                return region2
-
-        if operation == 'overwrite':
-            return region2
-        elif operation == 'inverse':
-            y = region1.dimension_values(1).copy()
-            y[region2.dimension_values(1) > 0] = 0
-            return region1.clone(data=(region1.edges, y))
-        elif operation == 'intersect':
-            op = np.min
-        elif operation == 'union':
-            op = np.max
-
-        return region1.clone(data=(
-            region1.edges,
-            op(np.stack([region1.dimension_values(1),
-                         region2.dimension_values(1)], axis=1), axis=1)
-        ))
-
-    def __setstate__(self, state):
-        """
-        Ensures old-style Histogram types without an interface can be unpickled.
-
-        Note: Deprecate as part of 2.0
-        """
-        if 'interface' not in state:
-            self.interface = GridInterface
-            x, y = state['_kdims_param_value'][0], state['_vdims_param_value'][0]
-            state['data'] = {x.name: state['data'][1], y.name: state['data'][0]}
-        super(Dataset, self).__setstate__(state)
-
-
-    @property
-    def values(self):
-        "Property to access the Histogram values provided for backward compatibility"
-        self.param.warning('Histogram.values is deprecated in favor of '
-                           'common dimension_values method.')
-        return self.dimension_values(1)
-
+        super().__init__(data, **params)
 
     @property
     def edges(self):
@@ -339,7 +220,6 @@ class Spikes(Selection1DExpr, Chart):
     _auto_indexable_1d = False
 
 
-
 class Area(Curve):
     """
     Area is a Chart element representing the area under a curve or
@@ -355,7 +235,7 @@ class Area(Curve):
     group = param.String(default='Area', constant=True)
 
     @classmethod
-    def stack(cls, areas):
+    def stack(cls, areas, baseline_name='Baseline'):
         """
         Stacks an (Nd)Overlay of Area or Curve Elements by offsetting
         their baselines. To stack a HoloMap or DynamicMap use the map
@@ -363,12 +243,22 @@ class Area(Curve):
         """
         if not len(areas):
             return areas
-        baseline = np.zeros(len(areas.values()[0]))
+        is_overlay = isinstance(areas, Overlay)
+        if is_overlay:
+            areas = NdOverlay({i: el for i, el in enumerate(areas)})
+        df = areas.dframe(multi_index=True)
+        levels = list(range(areas.ndims))
+        vdim = areas.last.vdims[0]
+        vdims = [vdim, baseline_name]
+        baseline = None
         stacked = areas.clone(shared_data=False)
-        vdims = [areas.values()[0].vdims[0], 'Baseline']
-        for k, area in areas.items():
-            x, y = (area.dimension_values(i) for i in range(2))
-            stacked[k] = area.clone((x, y+baseline, baseline), vdims=vdims,
-                                    new_type=Area)
-            baseline = baseline + y
-        return stacked
+        for key, sdf in df.groupby(level=levels, sort=False):
+            sdf = sdf.droplevel(levels).reindex(index=df.index.unique(-1), fill_value=0)
+            if baseline is None:
+                sdf[baseline_name] = 0
+            else:
+                sdf[vdim.name] = sdf[vdim.name] + baseline
+                sdf[baseline_name] = baseline
+            baseline = sdf[vdim.name]
+            stacked[key] = areas[key].clone(sdf, vdims=vdims)
+        return Overlay(stacked.values()) if is_overlay else stacked
