@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, unicode_literals
-
 from itertools import product
 
 import numpy as np
@@ -309,7 +307,7 @@ class RadialHeatMapPlot(ColorbarPlot):
     def get_data(self, element, ranges, style):
         # dimension labels
         dim_labels = element.dimensions(label=True)[:3]
-        x, y, z = [dimension_sanitizer(d) for d in dim_labels]
+        x, y, z = (dimension_sanitizer(d) for d in dim_labels)
 
         if self.invert_axes: x, y = y, x
 
