@@ -14,29 +14,30 @@ from matplotlib.patches import Path, PathPatch
 from matplotlib.transforms import Bbox, TransformedBbox, Affine2D
 from matplotlib.rcsetup import (
     validate_fontsize, validate_fonttype, validate_hatch)
+from packaging.version import Version
 
 try:  # starting Matplotlib 3.4.0
     from matplotlib._enums import CapStyle as validate_capstyle
     from matplotlib._enums import JoinStyle as validate_joinstyle
-except:  # before Matplotlib 3.4.0
+except ImportError:  # before Matplotlib 3.4.0
     from matplotlib.rcsetup import (
     validate_capstyle, validate_joinstyle)
 
 try:
     from nc_time_axis import NetCDFTimeConverter, CalendarDateTime
     nc_axis_available = True
-except:
+except ImportError:
     from matplotlib.dates import DateConverter
     NetCDFTimeConverter = DateConverter
     nc_axis_available = False
 
 from ...core.util import (
-    LooseVersion, arraylike_types, cftime_types, is_number
+    arraylike_types, cftime_types, is_number
 )
 from ...element import Raster, RGB, Polygons
 from ..util import COLOR_ALIASES, RGB_HEX_REGEX
 
-mpl_version = LooseVersion(matplotlib.__version__)
+mpl_version = Version(matplotlib.__version__)
 
 
 def is_color(color):
@@ -85,7 +86,7 @@ def get_old_rcparams():
     ]
     old_rcparams = {
         k: v for k, v in matplotlib.rcParams.items()
-        if mpl_version < LooseVersion('3.0') or k not in deprecated_rcparams
+        if mpl_version < Version('3.0') or k not in deprecated_rcparams
     }
     return old_rcparams
 
@@ -122,7 +123,7 @@ def validate(style, value, vectorized=True):
     try:
         valid = validator(value)
         return False if valid == False else True
-    except:
+    except Exception:
         return False
 
 
@@ -283,7 +284,7 @@ def fix_aspect(fig, nrows, ncols, title=None, extra_artists=[],
         bbox = get_tight_bbox(fig, extra_artists)
         top = bbox.intervaly[1]
         if title and title.get_text():
-            title.set_y((top/(w*aspect)))
+            title.set_y(top/(w*aspect))
 
 
 def get_tight_bbox(fig, bbox_extra_artists=[], pad=None):
@@ -308,8 +309,11 @@ def get_tight_bbox(fig, bbox_extra_artists=[], pad=None):
                 clip_path = clip_path.get_fully_transformed_path()
                 bbox = Bbox.intersection(bbox,
                                          clip_path.get_extents())
-        if bbox is not None and (bbox.width != 0 or
-                                 bbox.height != 0):
+        if (
+            bbox is not None and
+            (bbox.width != 0 or bbox.height != 0) and
+            np.isfinite(bbox).all()
+        ):
             bbox_filtered.append(bbox)
     if bbox_filtered:
         _bbox = Bbox.union(bbox_filtered)
